import java.awt.*;
class Main extends Frame {
    String[] colorNames = {"red", "green", "blue"};
    Color[] colorValues = {Color.red, Color.green, Color.blue};
    MainButton[] buttons = new MainButton[colorNames.length];
    Canvas cv = new Canvas();

    Main() {
        super("Canvas Example");
        Panel p = new Panel();

        p.setLayout(new GridLayout(1, 0));
        for (int i=0; i<colorNames.length; i++) {
            p.add(buttons[i] = new MainButton(colorNames[i], colorValues[i]));
        }
        add("South", p);
        cv.resize(150, 150);
        add("Center", cv);
        pack();
        show();
    }

    public boolean action(Event evt, Object arg) {
        if (evt.target instanceof MainButton) {
            for (int i=0; i<colorNames.length; i++) {
                if (colorNames[i].equals(arg)) {
                    buttons[i].disable();
                    cv.setBackground(colorValues[i]);
                    cv.repaint();
                } else {
                    buttons[i].enable();
                }
            }
            return true;
        }
        return false;
    }

    static public void main(String[] args) {
        new Main();
    }
}

class MainButton extends Canvas implements Runnable {
    boolean on;
    boolean engaged;
    String label;
    Thread timerThread;

    MainButton(String label, Color color) {
        this.label = label;
        (timerThread = new Thread(this)).start();
        resize(40, 40);
        setForeground(color);
    }

    public void enable() {
        super.enable();
        if (timerThread == null) {
            (timerThread = new Thread(this)).start();
        }
        repaint();
    }

    public void disable() {
        super.disable();
        timerThread = null;
        on = false;
        repaint();
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = size().width;
        int h = size().height;
        int ovalSize = Math.min(w/3, h/3);

        g.clearRect(0, 0, w, h);
        if (isEnabled()) {
            if (engaged || on) {
                g.fillOval((w-ovalSize)/2, (h-ovalSize)/2, 
                    ovalSize, ovalSize);
            } else {
                g.drawOval((w-ovalSize)/2, (h-ovalSize)/2, 
                    ovalSize, ovalSize);
            }
        } else {
            g.setColor(Color.white);
            g.drawOval((w-ovalSize)/2, (h-ovalSize)/2, ovalSize, ovalSize);
        }
        g.setColor(getBackground());
        g.draw3DRect(0, 0, w-1, h-1, !engaged);
        g.draw3DRect(1, 1, w-2, h-2, !engaged);
    }

    public boolean mouseDown(Event evt, int x, int y) {
        engaged = true;
        repaint();
        return true;
    }

    public boolean mouseUp(Event evt, int x, int y) {
        engaged = false;
        repaint();
        postEvent(new Event(this, Event.ACTION_EVENT, label));
        return true;
    }

    public void run() {
        while (timerThread == Thread.currentThread()) {
            on = !on;
            repaint();
            try { Thread.sleep(1000); } catch (Exception e) {};
        }
    }
}
