import java.awt.*;

class Main extends Frame {
    Scrollbar sbRed = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbGreen = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbBlue = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbH = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbS = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbB = new Scrollbar(Scrollbar.VERTICAL);
    TextField status = new TextField();
    int r = 256;
    int g = 256;
    int b = 256;
    float[] hsb = new float[3];

    Main() {
        super("Color Example");
        add("West", makeScrollbars("R", sbRed, "G", sbGreen, "B", sbBlue));
        add("East", makeScrollbars("H", sbH, "S", sbS, "B", sbB));
        status.setEditable(false);
        add("South", status);
        resize(300, 300);
        show();
        setBackground(adjustHSBScrollbars());
        adjustRGBScrollbars();
    }

    Panel makeScrollbars(String l1, Scrollbar sb1,
            String l2, Scrollbar sb2, String l3, Scrollbar sb3) {
        double[] rowWeights = {0.0, 1.0};
        Panel p = new Panel();
        GridBagLayout gbl = new GridBagLayout();

        gbl.rowWeights = rowWeights;
        p.setLayout(gbl);
        add(p, gbl, new Label(l1, Label.CENTER), 
            0, 0, GridBagConstraints.NONE);
        add(p, gbl, sb1, 0, 1, GridBagConstraints.VERTICAL);
        add(p, gbl, new Label(l2, Label.CENTER), 
            1, 0, GridBagConstraints.NONE);
        add(p, gbl, sb2, 1, 1, GridBagConstraints.VERTICAL);
        add(p, gbl, new Label(l3, Label.CENTER), 
            2, 0, GridBagConstraints.NONE);
        add(p, gbl, sb3, 2, 1, GridBagConstraints.VERTICAL);
        return p;
    }

    void add(Panel p, GridBagLayout gbl, Component comp, int x, int y, int fill) {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.fill = fill;
        gbl.setConstraints(comp, gbc);
        p.add(comp);
    }

    public boolean handleEvent(Event evt) {
        Color c = getBackground();

        if (evt.target == sbRed) {
            r = 255-sbRed.getValue();
            c = adjustHSBScrollbars();
        } else if (evt.target == sbGreen) {
            g = 255-sbGreen.getValue();
            c = adjustHSBScrollbars();
        } else if (evt.target == sbBlue) {
            b = 255-sbBlue.getValue();
            c = adjustHSBScrollbars();
        } else if (evt.target == sbH) {
            hsb[0] = (1000-sbH.getValue()) / 1000.0f;
            c = adjustRGBScrollbars();
        } else if (evt.target == sbS) {
            hsb[1] = (1000-sbS.getValue()) / 1000.0f;
            c = adjustRGBScrollbars();
        } else if (evt.target == sbB) {
            hsb[2] = (1000-sbB.getValue()) / 1000.0f;
            c = adjustRGBScrollbars();
        }
        setBackground(c);
        repaint();
        status.setText("RGB("+r+","+g+","+b+"  /  "+
            "#"+Integer.toString(c.getRGB()&0xffffff, 16)+")"+
            "    HSB("+hsb[0]+","+hsb[1]+","+hsb[2]+")");
        return super.handleEvent(evt);
    }

    Color adjustHSBScrollbars() {
        hsb = Color.RGBtoHSB(r, g, b, null);
        sbH.setValues(1000-(int)(hsb[0] * 1000), 1, 0, 1000);
        sbS.setValues(1000-(int)(hsb[1] * 1000), 1, 0, 1000);
        sbB.setValues(1000-(int)(hsb[2] * 1000), 1, 0, 1000);
        return new Color(r, g, b);
    }

    Color adjustRGBScrollbars() {
        Color c = Color.getHSBColor(hsb[0], hsb[1], hsb[2]);

        // An alternate way of converting the HSB values to RGB:
        // Color c = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));

        sbRed.setValues(255-(r = c.getRed()), 1, 0, 255);
        sbGreen.setValues(255-(g = c.getGreen()), 1, 0, 255);
        sbBlue.setValues(255-(b = c.getBlue()), 1, 0, 255);
        return c;
    }

    public static void main(String[] args) {
        new Main();
    }
}
