import java.awt.*;
class Main extends Frame {
    Main() {
        super("Component Example");
        add("Center", new InputCanvas(30, 5));
        pack();
        show();
    }
    
    public boolean action(Event evt, Object arg) {
        System.out.println(evt);
        return true;
    }

    static public void main(String[] args) {
        new Main();
    }
}

class InputCanvas extends Canvas implements Runnable {
    int border;
    int length;
    Font font;
    FontMetrics fontM;
    char[] buffer;
    int bufferIx;

    boolean hasFocus;
    boolean cursorVisible;
    boolean cursorOn;
    int saveCursor;

    InputCanvas(int len, int bor) {
        border = bor;
        length = len;
        buffer = new char[len];
        font = getFont();
        if (font == null) {
            font = new Font("Dialog", Font.PLAIN, 20);
        }
        fontM = getFontMetrics(font);

        (new Thread(this)).start();
    }

    public void paint(Graphics g) {
        int y = (size().height-fontM.getHeight())/2;

        g.setFont(font);
        g.drawChars(buffer, 0, bufferIx, border, y + fontM.getAscent());

        // Draw blinking cursor.
        int x = fontM.charsWidth(buffer, 0, bufferIx) + border;
        int w = fontM.charWidth('c');
        if (hasFocus) {
            g.setColor(getForeground());
            g.fillRect(x, y, w, fontM.getHeight());
            if (cursorOn) {
                if (bufferIx < buffer.length) {
                    g.setColor(getBackground());
                    g.fillRect(x+2, y+2, w-4, fontM.getHeight()-4);
                }
            }
        }
    }

    public Dimension minimumSize() {
        int w = fontM.charWidth('m') * length;
        return new Dimension(w + 2*border, 
            fontM.getHeight() + 2*border);
    }

    public Dimension preferredSize() {
        return minimumSize();
    }

    public boolean mouseEnter(Event evt, int x, int y) {
        Component p = getParent();
        while (p != null && !(p instanceof Frame)) {
            p = getParent();
        }
        if (p != null) {
            saveCursor = ((Frame)p).getCursorType();
            ((Frame)p).setCursor(Frame.TEXT_CURSOR);
        }
        return false; // Allow subclass to receive event.
    }

    public boolean mouseExit(Event evt, int x, int y) {
        Component p = getParent();
        while (p != null && !(p instanceof Frame)) {
            p = getParent();
        }
        if (p != null) {
            ((Frame)p).setCursor(saveCursor);
        }
        return false; // Allow subclass to receive event.
    }

    public boolean mouseDown(Event evt, int x, int y) {
        requestFocus();
        return true;
    }

    public boolean gotFocus(Event evt, Object what) {
        hasFocus = true;
        repaint();
        return true;
    }

    public boolean lostFocus(Event evt, Object what) {
        hasFocus = false;
        repaint();
        return true;
    }

    public boolean keyDown(Event evt, int key) {
        if (key == 127 || key == 8) {
            if (bufferIx > 0) {
                bufferIx--;
                repaint();
            }
        } else if (key == 9) {    // tab
            nextFocus();
        } else if (key == 10) {   // return
            evt.id = Event.ACTION_EVENT;
            evt.arg = String.valueOf(buffer, 0, bufferIx);
            postEvent(evt);
        } else if (bufferIx < buffer.length) {
            buffer[bufferIx++] = (char)key;
            repaint();
        }
        return true;
    }
    
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000);
                cursorOn = !cursorOn;
                repaint();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
