import java.awt.*;
import java.awt.image.*;
class Main extends Frame {
    Main(String filename) {
        super("key Example");
        Image image = getToolkit().getImage(filename);

        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        try {
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }

        ImageCanvas c = new ImageCanvas(image, new ScaledFrame());
        add("Center", c);
        resize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    int imageW, imageH;
    Image image;
    Rectangle lens = new Rectangle(0, 0, 20, 20);
    ScaledFrame frame;

    ImageCanvas(Image i, ScaledFrame f) {
        image = i;
        frame = f;
        imageW = image.getWidth(null);
        imageH = image.getHeight(null);
    }

    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
        g.setColor(Color.black);
        g.drawRect(lens.x, lens.y, lens.width-1, lens.height);
    }

    public boolean keyDown(Event evt, int key) {
        Rectangle oldRect = lens.union(lens);
        int delta = evt.controlDown() ? 5 : 1;

        switch (key) {
        case Event.UP:
            if (evt.shiftDown()) {
                lens.resize(lens.width, lens.height-delta);
            } else {
                lens.translate(0, -delta);
            }
            break;
        case Event.DOWN:
            if (evt.shiftDown()) {
                lens.resize(lens.width, lens.height+delta);
           } else {
                lens.translate(0, delta);
            }
            break;
        case Event.LEFT:
            if (evt.shiftDown()) {
                lens.resize(lens.width-delta, lens.height);
            } else {
                lens.translate(-delta, 0);
            }
            break;
        case Event.RIGHT:
            if (evt.shiftDown()) {
                lens.resize(lens.width+delta, lens.height);
            } else {
                lens.translate(delta, 0);
            }
            break;
        case Event.PGUP:
            lens.translate(0, -lens.height);
            break;
        case Event.PGDN:
            lens.translate(0, lens.height);
            break;
        case Event.HOME:
            lens.move(0, 0);
            break;
        case Event.END:
            lens.move(imageW-lens.width, imageH-lens.height);
            break;
        case Event.F1:
            frame.rescale(image, lens);
            break;
        }
        if (!oldRect.equals(lens)) {
            lens.width = Math.max(1, lens.width);
            lens.height = Math.max(1, lens.height);
            lens.move(Math.min(imageW-lens.width, lens.x), Math.min(imageH-lens.height, lens.y));
            lens.move(Math.max(0, lens.x), Math.max(0, lens.y));
            repaint();
        }
        return true;
    }
}

class ScaledFrame extends Frame {
    Image image;

    ScaledFrame() {
        super("key Example");
        resize(200, 200);
        show();
    }

    void rescale(Image image, Rectangle r) {
        CropImageFilter imgf = new CropImageFilter(
            r.x, r.y, r.width, r.height);
        this.image = getToolkit().createImage(
            new FilteredImageSource(image.getSource(), imgf));
        repaint();
    }

    public void paint(Graphics g) {
        int w = size().width - insets().right - insets().left;
        int h = size().height - insets().bottom - insets().top;

        if (image != null) {
            g.drawImage(image, 0, 0, w, h, this);
        }
    }
}
