import java.awt.*;
class Main extends Frame {
    Scrollbar sbVer = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbHor = new Scrollbar(Scrollbar.HORIZONTAL);
    MainCanvas cv = new MainCanvas(sbVer, sbHor);

    Main() {
        super("Scrollbar Example");
        add("Center", cv);
        add("East", sbVer);
        add("South", sbHor);
        resize(200, 200);
        show();
    }

    public boolean handleEvent(Event evt) {
        if (evt.target == sbVer || evt.target == sbHor) {
            Scrollbar sb = (Scrollbar)evt.target;

            System.out.println(
                ((sb.getOrientation() == Scrollbar.VERTICAL) ?
                    "ver" : "hor") 
                + " li=" + sb.getLineIncrement()
                + " pi=" + sb.getPageIncrement()
                + " max=" + sb.getMaximum()
                + " vis=" + sb.getVisible());
            cv.sbChanged();
            return true;
        }
        return super.handleEvent(evt);
    }

    static public void main(String[] args) {
        new Main();
    }
}

// Creates a very large square grid.
// Each grid cell is labelled to help with navigation.
class MainCanvas extends Canvas {
    int gridSize = 1500;
    int gridCells = 10;
    int cellSize = gridSize/gridCells;
    int originX, originY;
    Scrollbar sbVer;
    Scrollbar sbHor;

    MainCanvas(Scrollbar sbVer, Scrollbar sbHor) {
        this.sbVer = sbVer;
        this.sbHor = sbHor;
        setFont(new Font("Courier", Font.BOLD, 36));
    }

    public void sbChanged() {
        originY = -sbVer.getValue();
        originX = -sbHor.getValue();
        repaint();
    }

    // If the size of the canvas changes, we need to update
    // the scrollbar values.
    public void layout() {
        super.layout();
        int w = size().width;
        int h = size().height;

        sbVer.setValues(sbVer.getValue(), h, sbVer.getMinimum(), gridSize);
        sbVer.setPageIncrement(Math.min(cellSize, h));
        sbVer.setLineIncrement(Math.min(cellSize, h/5));
        sbHor.setValues(sbHor.getValue(), w, sbHor.getMinimum(), gridSize);
        sbHor.setPageIncrement(Math.min(cellSize, w));
        sbHor.setLineIncrement(Math.min(cellSize, w/5));

        // This ensures that if the grid is larger than the viewable
        // area, you will never see space around the grid.
        originX = -Math.max(0, Math.min(gridSize-w, sbHor.getValue()));
        originY = -Math.max(0, Math.min(gridSize-h, sbVer.getValue()));
    }

    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();

        // Using translate() simplifies the painting code
        // since it doesn't have to worry about origin changes.
        g.translate(originX, originY);
        g.setColor(Color.red);
        for (int i=0; i<gridCells; i++) {
            int c = i * cellSize;
            g.drawLine(c, 0, c, gridSize);
            g.drawLine(0, c, gridSize, c);
        }
        g.drawRect(0, 0, gridSize-1, gridSize-1);
        for (int i=0; i<gridCells; i++) {
            for (int j=0; j<gridCells; j++) {
                String str = "" + i + "," + j;
                int x = i * cellSize;
                int y = j * cellSize;

                g.drawString(str, x+(cellSize-fm.stringWidth(str))/2, 
                    y+(cellSize-fm.getHeight())/2+fm.getAscent());
            }
        }
    }
}
