import java.io.IOException;
import java.io.FilterOutputStream;
import java.io.FileInputStream;
import java.io.OutputStream;

class LineNumberOutputStream extends FilterOutputStream {
    private int linenumber = 0;
    private boolean neednewline = true;
    public LineNumberOutputStream (OutputStream out) {
        super(out);
    }

    protected void newline() {
        String prefix = ++linenumber + "\t";
        byte[] p = new byte[prefix.length()];
        prefix.getBytes(0, prefix.length(), p, 0);
        try {
            out.write(p);
        } catch (IOException e) {
            System.err.println(e);
        }
    }

    public void write(int b) throws IOException {
        if (neednewline) {
            newline();
            neednewline = false;
        }
        out.write(b);
        if (b == '\n') {
            neednewline = true;
        }
    }
}
class Main {
    public static void main(String[] args) {
        if (args.length == 0) {
            System.err.println("java Main <input file>");
            System.exit(-1);
        }
        try {
            FileInputStream in = new FileInputStream(args[0]);
            LineNumberOutputStream out = 
                new LineNumberOutputStream(System.out);

            for (int c = in.read(); c > -1; c = in.read())
                out.write((char)c);
            out.flush();
            out.close();
        } catch (IOException e) {
            System.out.println(e);
        }
    }
}
