import java.util.Hashtable;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

// START main
// A class loader that always checks a special directory
// for class files before using the default class loader
class DirClassLoader extends ClassLoader {
    private Hashtable cache = new Hashtable();
    private File classdir;
    DirClassLoader(String dir) {
	super();
	classdir = new File(dir);
    }
    // load the class data from the class file in specified dir
    private byte[] loadClassData(String name) 
	throws ClassNotFoundException {
	File target = new File(classdir, name + ".class");
// COM
 	System.out.println("Loading " + name + " from " + target);
// COM
	byte[] data = null;
	try {
	    long bytecount = target.length();
	    if (bytecount > 0) {
		data = new byte[(int)bytecount];
		FileInputStream in = new FileInputStream(target);
		in.read(data);
		in.close();
	    }
	} catch (IOException e) {
        	throw new ClassNotFoundException(name);
	}
	return (data);
    }
    public synchronized Class loadClass(String name, boolean resolve)
	throws ClassNotFoundException {
	Class c = (Class) cache.get(name);
	if (c == null) {
	    byte[] data = loadClassData(name);
	    if (data != null) {
		c = defineClass(data, 0, data.length);
		cache.put(name, c = defineClass(data, 0, data.length));
		if (resolve && c != null)
		    resolveClass(c);
	    } else {
		// Try default System loader
		c = findSystemClass(name);
// COM
		System.err.println("System loading " + name);
// COM
	    }
	}
	if (c == null)
	    throw new ClassNotFoundException(name);
	return c;
    }
}
// END main

// START const
class DebugClassLoader extends ClassLoader {
    public synchronized Class loadClass(String name, boolean resolve)
	throws ClassNotFoundException {
	    System.out.println("Loading " + name);
	    return (findSystemClass(name));
	}
}
// COM
class testdebug {
    public static Class test(String className) throws ClassNotFoundException {
// COM
DebugClassLoader loader = new DebugClassLoader();
Class c = loader.loadClass(className, true);
// END const
return (c);
}
}

// START load1
class TestClassLoader extends ClassLoader {
    private byte[] loadClassData(String className) {
// UNCOM ...	
// COM
	return null;
// COM
    }
// END load1
// START define
    public synchronized Class loadClass(String className, boolean resolve)
	throws ClassNotFoundException {
	    byte[] data = loadClassData(className);
	    if (data != null) {
		Class c = defineClass(data, 0, data.length);
		if (resolve && c != null)
		    resolveClass(c);
		return (c);
	    } else
		throw new ClassNotFoundException(className);
	}
// END define
// START load2
}
// COM
class testtest {
    public static void test(String className) {

TestClassLoader loader = new TestClassLoader();
try {
    Class c = loader.loadClass(className, true);
} catch (ClassNotFoundException e) {
// UNCOM ...
}
// END load2
}
}


class examples {
    static Object printClassDetails(Class c) {
	String myName = c.getName();
	Class mySuper = c.getSuperclass();
	String myDesc = c.toString();
	Class ifs[] = c.getInterfaces();
	ClassLoader myLoader = c.getClassLoader();
	boolean isInterf = c.isInterface();

	System.out.println("Name of Class: " + myName);
	System.out.println("Superclass is: " + mySuper);
	System.out.println("Class Loader is : " + myLoader);
	System.out.println("Description is : " + myDesc);
	for (int i = 0; i < ifs.length; i++)
	    System.out.println("Interface[" + i + "] is " + ifs[i]);

	if (isInterf)
	    System.out.println(myName + " is an Interface");
	else
	    System.out.println(myName + " is a class");

	Object obj = null;
	try {
	    obj = c.newInstance();
	} catch (InstantiationException e) {
	    System.err.println(e);
	} catch (IllegalAccessException e) {
	    System.err.println(e);
	}
	return (obj);
    }

    public static void main(String[] args) {
	// create a loader that only picks up classes from
	// the current directory
	DirClassLoader loader = new 
	    DirClassLoader("/home/rosanna/java/demo/mytest");
	DebugClassLoader loader2 = new DebugClassLoader();
	try {
	    Class hclass0 = loader2.loadClass(args[0], true);
	    Object obj0 = printClassDetails(hclass0);
	    System.out.println("Got object: " + obj0);

	    testdebug.test(args[0]);

	    Class hclass = loader.loadClass(args[0], true);
	    Object obj = printClassDetails(hclass);
	    System.out.println("Got object: " + obj);
	} catch (ClassNotFoundException e) {
	    System.out.println(e);
	}
    }
}
